<?php

// Updated by yas 2016/06/23
// Updated by yas 2016/05/26
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Updated by yas 2016/05/23
// Ureated by yas 2015/06/08.
namespace Drupal\Tests\cloud_server_template\Functional;

use Drupal\Component\Utility\Random;

use Drupal\Tests\aws_cloud\Functional\Ec2\AwsCloudTestCase;

/**
 * Tests CloudServerTemplate.
 *
 * @group Cloud
 */
class CloudServerTemplateTest extends AwsCloudTestCase {

  const CLOUD_SERVER_TEMPLATES_REPEAT_COUNT = 3;

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = [
    'cloud',
    'aws_cloud',
    'cloud_server_template',
  ];

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'add cloud server template entities',
      'list cloud server template',
      'view published cloud server template entities',
      'view unpublished cloud server template entities',
      'edit cloud server template entities',
      'delete cloud server template entities',
      'access cloud server template revisions',
      'revert all cloud server template revisions',
      'delete all cloud server template revisions',
    ];
  }

  /**
   * Tests CRUD for server_template information.
   */
  public function testCloudServerTemplate() {
    $this->createImage();

    $cloud_context = $this->cloud_context;

    // List Server Template for AWS.
    $this->drupalGet("/clouds/design/server_template/list/$cloud_context");
    $this->assertResponse(200, t('HTTP 200: List | Server Template'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));

    $vpcs = $this->createRandomVpcs();
    $subnets = $this->createRandomSubnets();
    $this->updateVpcsAndSubnetsToMockData($vpcs, $subnets);

    // Add a new server_template information.
    $add = $this->createServerTemplateTestData();
    for ($i = 0; $i < self::CLOUD_SERVER_TEMPLATES_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $vpc_index = array_rand($vpcs);
      $add[$i]['field_vpc'] = $vpcs[$vpc_index]['VpcId'];
      $vpc_name = $this->getNameFromArray($vpcs, $vpc_index, 'VpcId');

      $subnet_index = array_rand($subnets);
      $add[$i]['field_subnet'] = $subnets[$subnet_index]['SubnetId'];
      $subnet_name = $this->getNameFromArray($subnets, $subnet_index, 'SubnetId');

      $this->drupalPostForm("/clouds/design/server_template/add/$cloud_context/aws_cloud",
                            $add[$i],
                            t('Save'));
      $this->assertResponse(200, t('HTTP 200: Add | A New CloudServerTemplate Form #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
      $this->assertText(t('Created the @name Cloud Server Template', [
                          '@name' => $add[$i]['name[0][value]'],
                        ]),
                        t('Confirm Message') . ': '
                        . t('Created the @name Cloud Server Template', [
                          '@name' => $add[$i]['name[0][value]'],
                        ]));
      $this->assertText($add[$i]['name[0][value]']);
      $this->assertText($vpc_name);
      $this->assertText($subnet_name);

      // Make sure listing.
      $this->drupalGet("/clouds/design/server_template/list/$cloud_context");
      $this->assertResponse(200, t('HTTP 200: List | Server Template #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$j]['name[0][value]'],
                        t("Make sure w/ Listing @num: @name", [
                          '@num' => $j + 1,
                          '@name' => $add[$j]['name[0][value]'],
                        ]));
      }
    }

    // Edit case.
    $edit = $this->createServerTemplateTestData();
    for ($i = 0; $i < self::CLOUD_SERVER_TEMPLATES_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      $vpc_index = array_rand($vpcs);
      $edit[$i]['field_vpc'] = $vpcs[$vpc_index]['VpcId'];
      $vpc_name = $this->getNameFromArray($vpcs, $vpc_index, 'VpcId');

      $subnet_index = array_rand($subnets);
      $edit[$i]['field_subnet'] = $subnets[$subnet_index]['SubnetId'];
      $subnet_name = $this->getNameFromArray($subnets, $subnet_index, 'SubnetId');

      $this->drupalPostForm("/clouds/design/server_template/edit/$cloud_context/$num",
                            $edit[$i],
                            t('Save'));
      $this->assertResponse(200, t('HTTP 200: Edit | A New Server Template #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
      $this->assertText(t('Saved the @name Cloud Server Template.', [
                            '@name' => $edit[$i]['name[0][value]'],
                          ]),
                          t('Confirm Message') . ': '
                        . t('Saved the @name Cloud Server Template.', [
                          '@name' => $edit[$i]['name[0][value]'],
                        ]));
      $this->assertText($edit[$i]['name[0][value]']);
      $this->assertText($vpc_name);
      $this->assertText($subnet_name);

      // Make sure listing.
      $this->drupalGet("/clouds/design/server_template/list/$cloud_context");
      $this->assertResponse(200, t('HTTP 200: List | Server Template #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$j]['name[0][value]'],
                        t("Make sure w/ Listing @num: @name", [
                          '@num' => $j + 1,
                          '@name' => $edit[$j]['name[0][value]'],
                        ]));
      }
    }

    // Delete server_template Items
    // 3 times.
    for ($i = 0; $i < self::CLOUD_SERVER_TEMPLATES_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/design/server_template/delete/$cloud_context/$num");
      $this->drupalPostForm("/clouds/design/server_template/delete/$cloud_context/$num",
                            [],
                            t('Delete'));
      $this->assertResponse(200, t('HTTP 200: Server Template | Delete #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Make sure w/o Warnings'));

      // Make sure listing.
      $this->drupalGet("/clouds/design/server_template/list/$cloud_context");
      $this->assertResponse(200, t('HTTP 200: List | Server Template #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertNoText($edit[$j]['name[0][value]'],
                          t("Make sure w/ Listing @num: @name", [
                            '@num' => $j + 1,
                            '@name' => $edit[$j]['name[0][value]'],
                          ]));
      }
    }
  }

  /**
   * Tests CRUD for server_template revision information.
   */
  public function testCloudServerTemplateRevision() {
    $cloud_context = $this->cloud_context;

    $this->createImage();

    $vpcs = $this->createRandomVpcs();
    $subnets = $this->createRandomSubnets();
    $this->updateVpcsAndSubnetsToMockData($vpcs, $subnets);

    // Create a server template.
    $add = $this->createServerTemplateTestData();
    $this->drupalPostForm("/clouds/design/server_template/add/$cloud_context/aws_cloud",
                          $add[0],
                          t('Save'));
    $this->assertResponse(200, t('HTTP 200: Add | A New CloudServerTemplate Form #@num', ['@num' => 1]));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertText(t('Created the @name Cloud Server Template', [
                        '@name' => $add[0]['name[0][value]'],
                      ]),
                      t('Confirm Message') . ': '
                      . t('Created the @name Cloud Server Template', [
                        '@name' => $add[0]['name[0][value]'],
                      ]));

    // Make sure listing revisions.
    $this->drupalGet("/clouds/design/server_template/$cloud_context/1/revisions");
    $this->assertResponse(200, t('HTTP 200: List | Server Template Revision'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));

    // Create a new revision.
    $edit = $add[0];
    $old_description = $edit['field_description[0][value]'];
    $revision_desc = $this->random->name(32, TRUE);
    $edit['field_description[0][value]'] = $revision_desc;
    $edit['new_revision'] = '1';
    $this->drupalPostForm("/clouds/design/server_template/edit/$cloud_context/1",
                          $edit,
                          t('Save'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertText(t('Saved the @name Cloud Server Template.', [
                          '@name' => $edit['name[0][value]'],
                        ]),
                        t('Confirm Message') . ': '
                      . t('Saved the @name Cloud Server Template.', [
                        '@name' => $edit['name[0][value]'],
                      ]));

    // Make sure listing revisions.
    $this->drupalGet("/clouds/design/server_template/$cloud_context/1/revisions");
    $this->assertResponse(200, t('HTTP 200: List | Server Template Revision'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertSession()->linkByHrefExists("server_template/$cloud_context/1/revisions/1/view");
    $this->assertSession()->linkByHrefExists("server_template/$cloud_context/1/revisions/1/revert");
    $this->assertSession()->linkByHrefExists("server_template/$cloud_context/1/revisions/1/delete");

    // View the revision.
    $this->drupalGet("/clouds/design/server_template/$cloud_context/1/revisions/1/view");
    $this->assertResponse(200, t('HTTP 200: View | Server Template Revision'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertText($old_description);

    // Revert the revision.
    $this->drupalPostForm("/clouds/design/server_template/$cloud_context/1/revisions/1/revert",
                          [],
                          t('Revert'));
    $this->assertResponse(200, t('HTTP 200: List | Server Template Revision'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertText(t('Cloud Server Template @name has been reverted', [
                        '@name' => $edit['name[0][value]'],
                      ]));
    // A new revision is created.
    $this->assertSession()->linkByHrefExists("server_template/$cloud_context/1/revisions/2/view");

    // Delete the revision.
    $this->drupalPostForm("/clouds/design/server_template/$cloud_context/1/revisions/1/delete",
                          [],
                          t('Delete'));
    $this->assertResponse(200, t('HTTP 200: List | Server Template Revision'));
    $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
    $this->assertNoText(t('Warning'), t('Make sure w/o Warnings'));
    $this->assertText(t('Cloud Server Template @name has been deleted.', [
                        '@name' => $edit['name[0][value]'],
                      ]));
    // The revision is deleted.
    $this->assertSession()->linkByHrefNotExists("server_template/$cloud_context/1/revisions/1/view");
  }

  private function createServerTemplateTestData() {
    $random = $this->random;

    // 3 times.
    for ($i = 0; $i < self::CLOUD_SERVER_TEMPLATES_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[] = [
        'cloud_context[0][value]'       => $this->cloud_context,
        'name[0][value]'                => "Template #$num - " . date('Y/m/d') . $random->name(16, TRUE),
        'field_description[0][value]'   => "#$num: " . date('Y/m/d H:i:s - D M j G:i:s T Y')
                                           . ' - SimpleTest Server Template Description - '
                                           . $random->name(32, TRUE),
        'field_test_only[value]'        => '1',
        'field_instance_type'           => "m$num.xlarge",
        'field_availability_zone'       => 'us-west-1',
        'field_monitoring[value]'       => '1',
        'field_image_id'                => 1,
        'field_min_count[0][value]'     => 1,
        'field_max_count[0][value]'     => 1,
        'status[value]'                 => '1',
        'field_kernel_id[0][value]'     => 'aki-' . $random->name(8, TRUE),
        'field_ram[0][value]'           => 'ari-' . $random->name(8, TRUE),
      ];
    }
    return $data;
  }

  private function createImage() {
    $random = $this->random;

    // Create image.
    $image_id = 'ami-' . $random->name(6);
    $image = entity_create('aws_cloud_image', []);
    $image->setCloudContext($this->cloud_context);
    $image->set('ami_name', 'image1');
    $image->setImageId($image_id);
    $image->save();
  }

  private function createRandomVpcs() {
    $random = $this->random;

    $vpcs = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $vpcs[] = [
        'VpcId' => 'vpc-' . $random->name(8, TRUE),
        'Tags' => [
          ['Key' => 'Name', 'Value' => $random->name(10, TRUE)],
        ],
      ];

      $vpcs[] = [
        'VpcId' => 'vpc-' . $random->name(8, TRUE),
      ];
    }

    return $vpcs;
  }

  private function createRandomSubnets() {
    $random = $this->random;

    $subnets = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $subnets[] = [
        'SubnetId' => 'subnet-' . $random->name(8, TRUE),
        'Tags' => [
          ['Key' => 'Name', 'Value' => $random->name(10, TRUE)],
        ],
      ];

      $subnets[] = [
        'SubnetId' => 'subnet-' . $random->name(8, TRUE),
      ];
    }

    return $subnets;
  }

  private function updateVpcsAndSubnetsToMockData($vpcs, $subnets) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeVpcs']['Vpcs'] = $vpcs;
    $mock_data['DescribeSubnets']['Subnets'] = $subnets;
    $this->updateMockDataToConfig($mock_data);
  }

  private function getNameFromArray($array, $index, $id_key_name) {
    // Get id.
    $name = $array[$index][$id_key_name];

    // Get name if Tags exists.
    if (isset($array[$index]['Tags'])) {
      $name = sprintf('%s (%s)', $array[$index]['Tags'][0]['Value'], $name);
    }

    return $name;
  }

}
