<?php

/**
 * @file
 * Contains \Drupal\aws_cloud\Form\Config\AwsCloudAdminSettings.
 */

// Created by yas 2016/06/02.
namespace Drupal\aws_cloud\Form\Config;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class AWS Cloud Admin Settings.
 */
class AwsCloudAdminSettings extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'aws_cloud_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['aws_cloud.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $config = \Drupal::config('aws_cloud.settings');

    $form['aws_cloud_test_mode'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable test mode?'),
      '#default_value' => $config->get('aws_cloud_test_mode'),
      '#description' => t('This enables you to test the AWS Cloud module settings without accessing AWS.'),
    ];
    $form['aws_cloud_instance_types'] = [
      '#type' => 'textarea',
      '#title' => t('AWS Instance Types'),
      '#default_value' => $config->get('aws_cloud_instance_types'),
      '#description' => t('This is a list of AWS instance types. Enter them one per line'),
    ];
    $form['aws_cloud_notification'] = [
      '#type' => 'checkbox',
      '#title' => t('Enable instance notification'),
      '#description' => t('When enabled, instance owners will be notified if their instance has been running for too long.'),
      '#default_value' => $config->get('aws_cloud_notification'),
    ];
    $form['aws_cloud_notification_subject'] = [
      '#type' => 'textfield',
      '#title' => t('Email subject'),
      '#description' => t('Edit the email subject. Available variables are: [aws_cloud_instance:name], [aws_cloud_instance:id], [aws_cloud_instance:launch_time], [aws_cloud_instance:instance_state], [aws_cloud_instance:availability_zone], [aws_cloud_instance:private_ip], [aws_cloud_instance:public_up], [aws_cloud_instance:elastic_ip], [aws_cloud_instance:instance_link], [aws_cloud_instance:instance_link_edit]'),
      '#default_value' => $config->get('aws_cloud_notification_subject'),
    ];
    $form['aws_cloud_notification_msg'] = [
      '#type' => 'textarea',
      '#title' => t('Email message'),
      '#default_value' => $config->get('aws_cloud_notification_msg'),
      '#description' => t('Edit the email message. Available variables are: [aws_cloud_instance:name], [aws_cloud_instance:id], [aws_cloud_instance:launch_time], [aws_cloud_instance:instance_state], [aws_cloud_instance:availability_zone], [aws_cloud_instance:private_ip], [aws_cloud_instance:public_up], [aws_cloud_instance:elastic_ip], [aws_cloud_instance:instance_link], [aws_cloud_instance:instance_link_edit]')
    ];
    $form['aws_cloud_notification_frequency'] = [
      '#type' => 'select',
      '#options' => [
        86400 => t('Once a day'),
        604800 => t('Once every 7 days'),
        2592000 => t('Once every 30 days'),
      ],
      '#title' => t('Notification frequency'),
      '#description' => t('Instance owners will be notified once per option selected'),
      '#default_value' => $config->get('aws_cloud_notification_frequency')
    ];
    $form['aws_cloud_notification_criteria'] = [
      '#type' => 'select',
      '#options' => [
        1 => t('1 day'),
        30 => t('30 days'),
        60 => t('60 days'),
        90 => t('90 days'),
      ],
      '#title' => t('Notification criteria'),
      '#description' => t('Notify instance owners after an instance has been running for this period of time'),
      '#default_value' => $config->get('aws_cloud_notification_criteria')
    ];
    $form['aws_cloud_instance_terminate'] = [
      '#type' => 'checkbox',
      '#title' => t('Automatically terminate instance'),
      '#description' => t('Terminate instance automatically.'),
      '#default_value' => $config->get('aws_cloud_instance_terminate'),
    ];

    $form['aws_cloud_scheduler_tag'] = [
      '#type' => 'textfield',
      '#title' => t('Schedule Tag'),
      '#description' => t('Name of scheduling tag. This tag value is defined when setting up the <a href=":stack">AWS Instance Scheduler</a>', [
        ':stack' => 'https://docs.aws.amazon.com/solutions/latest/instance-scheduler/deployment.html'
      ]),
      '#default_value' => $config->get('aws_cloud_scheduler_tag'),
    ];

    $form['aws_cloud_scheduler_periods'] = [
      '#type' => 'textarea',
      '#title' => t('Schedule periods'),
      '#description' => t('<p>Schedules defined in AWS Instance Scheduler. The values entered are shown in the Schedule field on instance edit form and server template launch form. Enter one value per line, in the format <strong>key|label</strong>.</p><p>The key corresponds to the schedule name defined in AWS Instance Scheduler. The label is a free form descriptive value shown to users. An example configuration might be:<br/>office-hours|Office Hours - Monday to Friday 9:00am - 5:00pm.<br/><p>See <a href=:stack>Scheduler Configuration</a> for more information.</p>', [
        ':stack' => 'https://docs.aws.amazon.com/solutions/latest/instance-scheduler/components.html'
      ]),
      '#default_value' => $config->get('aws_cloud_scheduler_periods'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $config = \Drupal::configFactory()->getEditable('aws_cloud.settings');
    $form_state->cleanValues();

    foreach ($form_state->getValues() as $key => $value) {
      $config->set($key, Html::escape($value));
    }
    $config->save();
  }

}
