<?php

// Updated by yas 2016/05/31.
// Created by yas 2016/05/30.
namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Datetime\Element\Datetime;
use Drupal\Core\Datetime\Entity\DateFormat;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\Language;


/**
 * Form controller for the CloudScripting entity edit forms.
 *
 * @ingroup aws_cloud
 */
class InstanceEditForm extends AwsCloudContentForm {

  const SECURITY_GROUP_DELIMITER = ', ';

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {
    /* @var $entity \Drupal\aws_cloud\Entity\Ec2\Instance */
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;

    $form['cloud_context'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Cloud ID'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => !$entity->isNew()
      ? $entity->getCloudContext()
      : $cloud_context,
      '#required'      => FALSE,
      '#weight'        => -5,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,

    ];

    $form['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => FALSE,
      '#weight'        => -5,
    ];

    $form['image_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('EC2 Image'),
      '#size'          => 60,
      '#default_value' => $entity->getImageId(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['key_pair_name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Key Pair Name'),
      '#size'          => 60,
      '#default_value' => $entity->getKeyPairName(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['is_monitoring'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Monitoring Enabled'),
      '#size'          => 60,
      '#default_value' => $entity->isMonitoring(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['availability_zone'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Availability Zone'),
      '#size'          => 60,
      '#default_value' => $entity->getAvailabilityZone(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['security_groups'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Security Groups'),
      '#default_value' => explode(self::SECURITY_GROUP_DELIMITER, $entity->getSecurityGroups()),
      '#weight'        => -5,
      '#required'      => TRUE,
      '#multiple'      => TRUE,
      '#options'       => $this->getSecurityGroupsOptions(),
    ];

    $form['instance_type'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Instance Type'),
      '#default_value' => $entity->getInstanceType(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#options'       => $this->getInstanceTypeOptions(),
    ];

    if ($entity->getInstanceState() !== 'stopped') {
      $form['instance_type'] += [
        '#attributes'    => ['readonly' => 'readonly'],
        '#disabled'      => TRUE,
      ];
    }

    $form['kernel_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Kernel Image'),
      '#size'          => 60,
      '#default_value' => $entity->getKernelId(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['ramdisk_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Ramdisk Image'),
      '#size'          => 60,
      '#default_value' => $entity->getRamdiskId(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['user_data'] = [
      '#type'          => 'textarea',
      '#title'         => $this->t('User Data'),
      '#maxlength'     => 4096,
      '#cols'          => 60,
      '#rows'          => 3,
      '#default_value' => $entity->getUserData(),
      '#weight'        => -5,
      '#required'      => FALSE,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => TRUE,
    ];

    $form['login_username'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Login Username'),
      '#size'          => 60,
      '#default_value' => $entity->getLoginUsername() ?: 'ec2-user',
      '#weight'        => -5,
      '#required'      => FALSE,
    ];

    $form['langcode'] = [
      '#title'         => t('Language'),
      '#type'          => 'language_select',
      '#default_value' => $entity->getUntranslated()->language()->getId(),
      '#languages'     => Language::STATE_ALL,
      '#attributes'    => ['readonly' => 'readonly'],
      '#disabled'      => FALSE,
    ];

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);

    // Set a message for termination timestamp.
    $date_format = DateFormat::load('html_date')->getPattern();
    $time_format = DateFormat::load('html_time')->getPattern();
    $form['termination_timestamp']['widget'][0]['value']['#description'] = $this->t('Format: %format. Leave blank for no automatic termination.', ['%format' => Datetime::formatExample($date_format . ' ' . $time_format)]);;

    $schedule = $entity->getSchedule();
    $form['schedule'] = [
      '#title' => t('Schedule'),
      '#type' => 'select',
      '#default_value' => isset($schedule) ? $entity->getSchedule() : '',
      '#options' => aws_cloud_get_schedule(),
      '#weight' => -4,
      '#description' => t('Specify a start/stop schedule. This helps reduce server hosting costs.'),
    ];

    return $form;
  }

  /**
   * {@inheritdocs}
   */
  public function save(array $form, FormStateInterface $form_state) {
    /* @var \Drupal\Core\Datetime\DrupalDateTime $termination_timestamp */
    $termination_timestamp = $form_state->getValue('termination_timestamp')[0]['value'];
    if ($termination_timestamp == NULL) {
      // Unset the termination timestamp.
      $this->entity->set('termination_timestamp', NULL);
    }

    $security_groups = array_values($form_state->getValue('security_groups'));
    if ($security_groups == NULL) {
      $security_groups = [];
    }
    $this->entity->setSecurityGroups(implode(self::SECURITY_GROUP_DELIMITER, $security_groups));

    parent::save($form, $form_state);

    // update the instance name - it is a tag.
    $params = [
      'Resources' => [$this->entity->get('instance_id')->value],
      'Tags' => [
        [
          'Key' => 'Name',
          'Value' => $this->entity->get('name')->value
        ],
        [
          'Key' => 'cloud_launched_by_uid',
          'Value' => $this->entity->getOwner()->id()
        ],
      ]
    ];

    $params['Tags'][] = [
      'Key' => 'cloud_termination_timestamp',
      'Value' => $termination_timestamp == NULL ? NULL : $termination_timestamp->getTimestamp(),
    ];

    // send the schedule if scheduler is enabled.
    $config = \Drupal::config('aws_cloud.settings');
    if ($config->get('aws_cloud_scheduler') == TRUE) {
      $params['Tags'][] = [
        'Key' => $config->get('aws_cloud_scheduler_tag'),
        'Value' => $this->entity->getSchedule()
      ];
    }

    $this->awsEc2Service->setCloudContext($this->entity->get('cloud_context')->value);
    $this->awsEc2Service->createTags($params);

    // Update the instance type, security group.
    $params = [];
    if ($this->entity->getInstanceState() == 'stopped'
      && $this->entity->getInstanceType() != $form['instance_type']['#default_value']) {

      $params['InstanceType'] = ['Value' => $this->entity->getInstanceType()];
    }

    if ($this->entity->getSecurityGroups() != implode(self::SECURITY_GROUP_DELIMITER, $form['security_groups']['#default_value'])) {
      $params['Groups'] = $this->getSecurityGroupIdsByNames($security_groups);
    }

    if (!empty($params)) {
      $params['InstanceId'] = $this->entity->get('instance_id')->value;

      $response = $this->awsEc2Service->modifyInstanceAttribute($params);
      if ($response['Return']) {
        $this->messenger->addMessage($this->t('Instance attributes was updated.'), 'status');
      }
      else {
        $this->messenger->addMessage($this->t('Instance attributes couldn\'t be updated.'), 'error');
      }
    }

    // Set the redirect url to the listing page.
    $form_state->setRedirect('view.aws_instances.page_1', ['cloud_context' => $this->entity->getCloudContext()]);
  }

  private function getInstanceTypeOptions() {
    $types = \Drupal::config('aws_cloud.settings')->get('aws_cloud_instance_types');
    $instance_array = explode("\r\n", $types);
    foreach ($instance_array as $type) {
      $instance_types[$type] = $type;
    }
    return $instance_types;
  }

  private function getSecurityGroupsOptions() {
    $options = [];
    $this->awsEc2Service->setCloudContext($this->entity->get('cloud_context')->value);
    $response = $this->awsEc2Service->describeSecurityGroups([
      'Filters' => [[
        'Name' => 'vpc-id',
        'Values' => [$this->entity->getVpcId()],
      ]]
    ]);

    foreach ($response['SecurityGroups'] as $security_group) {
      $options[$security_group['GroupName']] = $security_group['GroupName'];
       }

    asort($options);
    return $options;
  }

  private function getSecurityGroupIdsByNames($group_names) {
    $response = $this->awsEc2Service->describeSecurityGroups([
      'GroupNames' => $group_names,
    ]);

    $group_ids = [];
    foreach ($response['SecurityGroups'] as $security_group) {
      $group_ids[] = $security_group['GroupId'];
     }

    return $group_ids;
  }

}
