<?php

// Updated by yas 2016/09/11
// Updated by yas 2016/06/04
// Updated by yas 2016/05/31
// Updated by yas 2016/05/30
// Updated by yas 2016/05/25
// Updated by yas 2016/05/20
// Created by yas 2016/05/19.
namespace Drupal\aws_cloud\Form\Ec2;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Language\Language;

/**
 * Form controller for the Volume entity create form.
 *
 * @ingroup aws_cloud
 */
class VolumeCreateForm extends AwsCloudContentForm {

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::buildForm().
   */
  public function buildForm(array $form, FormStateInterface $form_state, $cloud_context = '') {

    $this->awsEc2Service->setCloudContext($cloud_context);

    /* @var $entity \Drupal\aws_cloud\Entity\Ec2\Volume */
    $form = parent::buildForm($form, $form_state);

    $entity = $this->entity;

    $form['cloud_context'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Cloud ID'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => !$entity->isNew()
        ? $entity->getCloudContext()
        : $cloud_context,
      '#required'      => TRUE,
      '#weight'        => -5,
      '#disabled'      => TRUE,
    ];

    $form['name'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Name'),
      '#maxlength'     => 255,
      '#size'          => 60,
      '#default_value' => $entity->label(),
      '#required'      => TRUE,
      '#weight'        => -5,
    ];

    $form['snapshot_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Snapshot ID'),
      '#size'          => 60,
      '#default_value' => $entity->getSnapshotId(),
      '#weight'        => -5,
      '#required'      => FALSE,
    ];

    $availability_zones = $this->awsEc2Service->getAvailabilityZones();

    $form['availability_zone'] = [
      '#type'          => 'select',
      '#title'         => $this->t('Availability Zone'),
      '#options'       => $availability_zones,
      // Pick up the first availability zone in the array.
      '#default_value' => array_shift($availability_zones),
      '#weight'        => -5,
      '#required'      => TRUE,
    ];

    $form['size'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('Size (GB)'),
      '#size'          => 60,
      '#default_value' => $entity->getSize(),
      '#weight'        => -5,
      '#required'      => FALSE,
    ];

    $form['iops'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('IOPS'),
      '#size'          => 60,
      '#default_value' => $entity->getIops(),
      '#weight'        => -5,
      '#required'      => FALSE,
    ];

    $form['volume_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Volume Type'),
      '#options' => [
        'standard' => $this->t('Magnetic (standard)'),
        'io1' => $this->t('Provisioned IOPS SSD (io1)'),
        'gp2' => $this->t('General Purpose SSD (gp2)'),
        'sc1' => $this->t('Cold HDD (sc1)'),
        'st1' => $this->t('Throughput Optimized HDD (st1)'),
      ],
      '#weight' => -4,
    ];

    $form['encrypted'] = [
      '#type'          => 'checkbox',
      '#title'         => $this->t('Encrypted'),
      '#size'          => 60,
      '#default_value' => $entity->getEncrypted(),
      '#weight'        => -5,
      '#required'      => FALSE,

    ];
    $form['kms_key_id'] = [
      '#type'          => 'textfield',
      '#title'         => $this->t('kms_key_id'),
      '#size'          => 60,
      '#default_value' => $entity->getKmsKeyId(),
      '#weight'        => -5,
      '#required'      => FALSE,
    ];

    $form['langcode'] = [
      '#title' => t('Language'),
      '#type' => 'language_select',
      '#default_value' => $entity->getUntranslated()->language()->getId(),
      '#languages' => Language::STATE_ALL,
    ];

    $form['actions'] = $this->actions($form, $form_state, $cloud_context);

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    // add validation for volume types.  for io1 type,
    // iops must be set.  For other volume types, iops cannot
    // be set.
    if ($form_state->getValue('volume_type') == 'io1') {
      // Check if there is an iops value.
      if (empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('Please specify an iops value.  The value must be a minimum of 100.'));
      }

      // Check if iops is an integer.
      if (!is_numeric($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be an integer.'));
      }
      // Check if iops is greater than 100.
      $iops = (int) $form_state->getValue('iops');
      if ($iops < 100) {
        $form_state->setErrorByName('iops', $this->t('IOPS must be a minimum of 100.'));
      }
    }
    else {
      if (!empty($form_state->getValue('iops'))) {
        $form_state->setErrorByName('iops', $this->t('IOPS cannot be set unless volume type is "Provisioned IOPS SSD".'));
      }
    }
  }

  /**
   * Overrides Drupal\Core\Entity\EntityFormController::save().
   */
  public function save(array $form, FormStateInterface $form_state) {

    $entity = $this->entity;

    $params = [
      'Size'             => $entity->getSize(),
      'SnapshotId'       => $entity->getSnapshotId(),
      'AvailabilityZone' => $entity->getAvailabilityZone(), // REQUIRED.
      'VolumeType'       => $entity->getVolumeType(),
      'Encrypted'        => $entity->getEncrypted() ? TRUE : FALSE,
    ];

    if ($entity->getVolumeType() == 'io1') {
      $params['Iops'] = (int) $entity->getIops();
    }

    if (!empty($entity->getKmsKeyId())) {
      $params['KmsKeyId'] = $entity->getKmsKeyId();
    }

    $result = $this->awsEc2Service->createVolume($params);

    if (isset($result['VolumeId'])
      && ($entity->setVolumeId($result['VolumeId']))
      && ($entity->setCreated($result['CreateTime']))
      && ($entity->setState($result['State']))
      && ($entity->save())) {

      $message = $this->t('The @label "%label (@volume_id)" has been created.', [
        '@label'     => $entity->getEntityType()->getLabel(),
        '%label'     => $entity->label(),
        '@volume_id' => $entity->getVolumeId(),
      ]);
      $this->messenger->addMessage($message);
      $form_state->setRedirect('view.aws_volume.page_1', ['cloud_context' => $entity->getCloudContext()]);
    }
    else {
      $message = $this->t('The @label "%label" couldn\'t create.', [
        '@label' => $entity->getEntityType()->getLabel(),
        '%label' => $entity->label(),
      ]);
      $this->messenger->addError($message);
    }

  }

}
