<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Component\Utility\Random;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Created by yas 2016/05/28.
define('AWS_CLOUD_IMAGE_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Image.
 *
 * @group AWS Cloud
 */
class ImageTest extends AwsCloudTestCase {
  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'add aws cloud image',
      'list aws cloud image',
      'view aws cloud image',
      'edit aws cloud image',
      'delete aws cloud image',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'image_id' => 'ami-' . $this->random->name(8, TRUE),
      'owner_id' => rand(100000000000, 999999999999), //12 digits
      'name' => $this->random->name(8, TRUE),
      'kernel_id' => 'aki-' . $this->random->name(8, TRUE),
      'ramdisk_id' => 'ari-' . $this->random->name(8, TRUE),
      'product_code1' => $this->random->name(8, TRUE),
      'product_code2' => $this->random->name(8, TRUE),
      'image_location' => $this->random->name(16, TRUE),
      'state_reason_message' => $this->random->name(8, TRUE),
      'platform' => $this->random->name(8, TRUE),
      'description' => $this->random->string(8, TRUE),
      'creation_date' => date('c'),
    ];
  }

  /**
   * Tests CRUD for image information.
   */
  public function testImage() {
    $cloud_context = $this->cloud_context;

    // List Image for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
    $this->assertResponse(200, t('List | HTTP 200: Image'));
    $this->assertNoText(t('Notice'), t('List | ake sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Register a new Image.
    $add = $this->createImageTestData();
    // 3 times.
    for ($i = 0; $i < AWS_CLOUD_IMAGE_REPEAT_COUNT; $i++) {
      $this->reloadMockData();
      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/image/add",
                            $add[$i],
                            t('Save'));
      $this->assertResponse(200, t('Add | HTTP 200: A New Cloud Image #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText($add[$i]['name'], t('Name: @name ', ['@name' => $add[$i]['name']]));
      $this->assertText(t('The AWS Cloud Image "@name', [
                          '@name' => $add[$i]['name'],
                        ]),
                        t('Confirm Message') . ': '
                        . t('Add | The AWS Cloud Image "@name" has been created.', [
                          '@name' => $add[$i]['name'],
                        ]));

      // Make sure View.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image/$num");
      $this->assertResponse(200, t('Add | View | HTTP 200: Image #@num', array('@num' => $num)));
      $this->assertNoText(t('Notice'), t('Add | View | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | View | Make sure w/o Warnings'));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertResponse(200, t('Add | List | HTTP 200: Image #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));

      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$j]['name'],
                        t('Add | List | Make sure w/ Listing: @name', [
                          '@name' => $add[$j]['name'],
                        ]));
      }
    }

    // Edit an Image information.
    $edit = $this->createImageTestData();
    for ($i = 0; $i < AWS_CLOUD_IMAGE_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      unset($edit[$i]['instance_id']);
      unset($edit[$i]['description']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/image/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('HTTP 200: Edit | A New Image #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Make sure w/o Warnings'));
      $this->assertText($add[$i]['name'], t('Name: @name ', ['@name' => $add[$i]['name']]));
      $this->assertText(t('The AWS Cloud Image "@name" has been saved.', ['@name' => $add[$i]['name']]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Image "@name" has been saved.', [
                        '@name' => $add[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Image #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $add[$i]['name'],
                        ]));
      }
    }

    // Delete Image.
    for ($i = 0; $i < AWS_CLOUD_IMAGE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->deleteImage($cloud_context, $num, $edit[$i]['name']);

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertResponse(200, t('Delete | List | HTTP 200: Image #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) { // 3 times
        $this->assertNoText($edit[$i]['name'],
          t('Delete | List | Make sure w/ Listing: @name', array(
          '@name' => $edit[$i]['name']
        )));
      }
    }
  }

  public function testImportImage() {
    $cloud_context = $this->cloud_context;
    $image_id = 'ami-' . $this->random->name(8, TRUE);
    $product_code1 = $this->random->name(8, TRUE);
    $product_code2 = $this->random->name(8, TRUE);
    $name = "Image " . date('Y/m/d - ') . $this->random->name(8, TRUE);
    $this->updateImageInMockData($image_id, $name, $product_code1, $product_code2);

    // Import image.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/images/import");
    $this->assertResponse(200, t('Import | HTTP 200: Image'));
    $this->assertNoText(t('Notice'), t('Import | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Import | Make sure w/o Warnings'));

    $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/images/import",
                          ['image_ids' => $image_id],
                          t('Import'));
    $this->assertResponse(200, t('Import | HTTP 200: A New Cloud Image'));
    $this->assertNoText(t('Notice'), t('Import | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Import | Make sure w/o Warnings'));
    $this->assertText('Imported 1 images', t('Imported 1 images'));
    $this->assertText($image_id, t('Image Id: ' . $image_id));

    $num = 1;

    // View image
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/image/$num");
    $this->assertResponse(200, t('Add | View | HTTP 200: Image #@num', array('@num' => $num)));
    $this->assertNoText(t('Notice'), t('Add | View | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Add | View | Make sure w/o Warnings'));
    $this->assertText($image_id, t('Image Id: ' . $image_id));
    $this->assertText("$product_code1,$product_code2",
                      t('Product code: ' . "$product_code1,$product_code2"));

    // Delete image
    $this->deleteImage($cloud_context, $num, $name);

    // Make sure listing.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
    $this->assertResponse(200, t('Delete | List | HTTP 200: Image #@num', ['@num' => $num]));
    $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));
    $this->assertNoText($name,
      t('Delete | List | Make sure w/ Listing: @name', array(
      '@name' => $name
    )));
  }

  private function deleteImage($cloud_context, $num, $name) {
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/image/$num/delete");
    $this->assertResponse(200, t('Delete | HTTP 200: Image #@num', ['@num' => $num]));
    $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
    $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/image/$num/delete",
                          [],
                          t('Delete'));

    $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Image #@num', ['@num' => $num]));
    $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
    $this->assertText($name, t('Name: @name ', ['@name' => $name]));
    $this->assertText(t('The AWS Cloud Image "@name" has been deleted.', [
                        '@name' => $name,
                      ]),
                        t('Confirm Message') . ': '
                      . t('Delete | The AWS Cloud Image "@name" has been deleted.', [
                        '@name' => $name,
                      ]));

  }

  private function createImageTestData() {
    $data = [];
    for ($i = 0; $i < AWS_CLOUD_IMAGE_REPEAT_COUNT; $i++) {
      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'        => "Image #$num - " . date('Y/m/d - ') . $this->random->name(8, TRUE),
        'instance_id' => 'i-' . $this->random->name(8),
        'description' => 'description-' . $this->random->name(64),
      ];
    }
    return $data;
  }

  private function updateImageInMockData($image_id, $name, $product_code1, $product_code2) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeImages']['Images'][0]['ImageId'] = $image_id;
    $mock_data['DescribeImages']['Images'][0]['Name'] = $name;
    $mock_data['DescribeImages']['Images'][0]['ProductCodes'][0]['ProductCode'] = $product_code1;
    $mock_data['DescribeImages']['Images'][0]['ProductCodes'][1]['ProductCode'] = $product_code2;
    $this->updateMockDataToConfig($mock_data);
  }

}
