<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

// Updated by yas 2016/09/07
// Updated by yas 2016/06/24
// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Updated by yas 2016/05/23
// Updated by yas 2016/05/22
// Created by yas 2016/05/21.
use Drupal\Component\Utility\Random;
use Drupal\Component\Serialization\Yaml;

use Drupal\Tests\aws_cloud\Functional\Utils;

define('AWS_CLOUD_INSTANCE_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Instance.
 *
 * @group AWS Cloud
 */
class InstanceTest extends AwsCloudTestCase {

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'add aws cloud instance',
      'list aws cloud instance',
      'edit own aws cloud instance',
      'delete own aws cloud instance',
      'view own aws cloud instance',

      'list cloud server template',
      'launch server template',

      'add aws cloud image',
      'list aws cloud image',
      'view aws cloud image',
      'edit aws cloud image',
      'delete aws cloud image',

      'administer aws_cloud',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    $public_ip = Utils::getRandomPublicIp();
    $private_ip = Utils::getRandomPrivateIp();
    $regions = ['us-west-1', 'us-west-2'];
    $region = $regions[array_rand($regions)];

    return [
      // 12 digits
      'owner_id' => rand(100000000000, 999999999999),
      'reservation_id' => 'r-' . $this->random->name(17, TRUE),
      'group_name' => $this->random->name(8, TRUE),
      'host_id' => $this->random->name(8, TRUE),
      'affinity' => $this->random->name(8, TRUE),
      'launch_time' => date('c'),
      'security_group_id' => 'sg-' . $this->random->name(8, TRUE),
      'security_group_name' => $this->random->name(10, TRUE),
      'public_dns_name' => Utils::getPublicDns($region, $public_ip),
      'public_ip_address' => $public_ip,
      'private_dns_name' => Utils::getPrivateDns($region, $private_ip),
      'private_ip_address' => $private_ip,
      'vpc_id' => 'vpc-' . $this->random->name(8, TRUE),
      'subnet_id' => 'subnet-' . $this->random->name(8, TRUE),
      'image_id' => 'ami-' . $this->random->name(8, TRUE),
      'reason' => $this->random->string(16, TRUE),
      'instance_id' => 'i-' . $this->random->name(17, TRUE),
      'state' => 'running',
    ];
  }

  /**
   * Tests CRUD for instance information.
   */
  public function testInstance() {
    $cloud_context = $this->cloud_context;
    $this->createServerTemplate();

    // List Instance for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
    $this->assertResponse(200, t('HTTP 200: List | Instance'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Launch a new Instance.
    $add = $this->createInstanceTestData();
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->addInstanceMockData($add[$i]['name'], $add[$i]['key_pair_name']);
      $this->drupalPostForm("/clouds/design/server_template/$cloud_context/1/launch",
                            [],
                            t('Launch'));
      $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));

      // Make sure listing.
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$j]['name'], t('Make sure w/ Listing: @name', [
          '@name' => $add[$j]['name'],
        ]));
      }
    }

    // Edit an Instance information.
    $edit = $this->createInstanceTestData();
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['cloud_type']);
      unset($edit[$i]['image_id']);
      unset($edit[$i]['min_count']);
      unset($edit[$i]['max_count']);
      unset($edit[$i]['key_pair_name']);
      unset($edit[$i]['is_monitoring']);
      unset($edit[$i]['availability_zone']);
      unset($edit[$i]['instance_type']);
      unset($edit[$i]['kernel_id']);
      unset($edit[$i]['ramdisk_id']);
      unset($edit[$i]['user_data']);

      // Change security groups.
      $security_groups = $this->createRandomSecurityGroups();
      $this->updateDescribeSecurityGroupsInMockData($security_groups);
      $edit[$i]['security_groups[]'] = [array_column($security_groups, 'GroupName')[0]];

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->updateInstanceInMockData($num - 1, $edit[$i]['name']);

      $this->assertResponse(200, t('Edit | HTTP 200:  A New Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $edit[$i]['name'],
                        ]));
      }
    }

    // Terminate Instance.
    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/$num/terminate");
      $this->assertResponse(200, t('Terminate: HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | Make sure w/o Warnings'));

      $this->deleteFirstInstanceInMockData();

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/terminate",
                            [],
                            t('Delete | Terminate'));

      $this->assertResponse(200, t('Terminate | HTTP 200: The Cloud Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Instance Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Instance "@name" has been terminated.', [
                          '@name' => $edit[$i]['name'],
                        ]),
                        t('Confirm Message') . ': '
                        . t('Terminate | The AWS Cloud Instance "@name" has been terminated.', [
                          '@name' => $edit[$i]['name'],
                        ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance");
      $this->assertResponse(200, t('Terminate | HTTP 200: Instance #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Terminate | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Terminate | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) { // 3 times
        $this->assertNoText($edit[$i]['name'],
                            t('Terminate | List | Make sure w/ Listing: @name', array(
                            '@name' => $edit[$i]['name']
                          )));
      }
    }
  }

  /**
   * Tests updating instances.
   */
  function testUpdateInstances() {
    $cloud_context = $this->cloud_context;
    $this->createServerTemplate();

    // Launch a new Instance.
    $add = $this->createInstanceTestData();
    $this->addInstanceMockData($add[0]['name'], $add[0]['key_pair_name']);
    $this->drupalPostForm("/clouds/design/server_template/$cloud_context/1/launch",
                          [],
                          t('Launch'));
    $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance 1'));
    $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));

    // Change security groups.
    $security_group_name1 = $this->random->name(8, TRUE);
    $security_group_name2 = $this->random->name(8, TRUE);
    $this->updateSecurityGroupsInMockData($security_group_name1, $security_group_name2);

    // Change instance type.
    $instance_type = $this->random->name(6, TRUE);
    $this->updateInstanceTypeInMockData($instance_type);

    // Update the schedule tag.
    $schedule_value = $this->random->name(8, TRUE);
    $this->updateScheduleTagInMockData($schedule_value);

    // Run cron job to update instances.
    $key = \Drupal::state()->get('system.cron_key');
    $this->drupalGet('/cron/' . $key);
    $this->assertResponse(204);

    // Verify schedule tag.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/1");
    $this->assertResponse(200, t('View | HTTP 200: The Cloud Instance 1'));
    $this->assertNoText(t('Notice'), t('View | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('View | Make sure w/o Warnings'));
    $this->assertText("$schedule_value", t('Schedule'));

    // Verify security group.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/1");
    $this->assertResponse(200, t('View | HTTP 200: The Cloud Instance 1'));
    $this->assertNoText(t('Notice'), t('View | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('View | Make sure w/o Warnings'));
    $this->assertText("$security_group_name1, $security_group_name2", t('Security Group'));
    $this->assertText($instance_type, t('Instance Type'));
  }

  /**
   * Tests updating instance attributes.
   */
  function testUpdateInstanceAttributes() {
    $this->_testUpdateInstanceAttributesRepeat(AWS_CLOUD_INSTANCE_REPEAT_COUNT);
  }

  private function _testUpdateInstanceAttributesRepeat($max_test_repeat_count = 1) {
    $cloud_context = $this->cloud_context;

    for ($i = 0; $i < $max_test_repeat_count; $i++) {
      $num = $i + 1;

      $this->createServerTemplate();

      // Launch a stopped Instance.
      $add = $this->createInstanceTestData();
      $this->addInstanceMockData($add[$i]['name'], $add[$i]['key_pair_name'], 'stopped');
      $this->drupalPostForm("/clouds/design/server_template/$cloud_context/$num/launch",
                            [],
                            t('Launch'));
      $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance @num', [
                                    '@num' => $num
                                  ]));
      $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));
      $this->assertText('stopped', t('Instance Type: stopped'));

      // Edit instance.
      $edit = $this->createInstanceTestData();
      unset($edit[$i]['cloud_type']);
      unset($edit[$i]['image_id']);
      unset($edit[$i]['min_count']);
      unset($edit[$i]['max_count']);
      unset($edit[$i]['key_pair_name']);
      unset($edit[$i]['is_monitoring']);
      unset($edit[$i]['availability_zone']);
      unset($edit[$i]['kernel_id']);
      unset($edit[$i]['ramdisk_id']);
      unset($edit[$i]['user_data']);

      $instance_type = 'm1.small';
      $edit[$i]['instance_type'] = $instance_type;

      // Change security groups.
      $security_groups = $this->createRandomSecurityGroups();
      $this->updateDescribeSecurityGroupsInMockData($security_groups);
      $edit[$i]['security_groups[]'] = array_column($security_groups, 'GroupName');

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->updateInstanceInMockData($i, $edit[$i]['name']);

      $this->assertResponse(200, t('Edit | HTTP 200:  A New Instance #@num', [
                                    '@num' => $num
                                  ]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Instance "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));
      $this->assertText('Instance attributes was updated.');

      // Verify instance attributes.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/$num");
      $this->assertResponse(200, t('View | HTTP 200: The Cloud Instance @num', [
                                    '@num' => $num
                                  ]));
      $this->assertNoText(t('Notice'), t('View | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('View | Make sure w/o Warnings'));
      $this->assertText($instance_type, t('Instance Type') . ': ' . $instance_type);
      $this->assertText(implode(', ', $edit[$i]['security_groups[]']),
                        t('Security Groups') . ': ' . implode(', ', $edit[$i]['security_groups[]'])
                        );
    }
  }

  /**
   * Tests setting the configuration of instance terminating.
   */
  function testInstanceTerminateConfiguration() {
    $this->_testInstanceTerminateConfigurationRepeat(AWS_CLOUD_INSTANCE_REPEAT_COUNT);
  }

  public function _testInstanceTerminateConfigurationRepeat($max_test_repeat_count = 1) {
    $cloud_context = $this->cloud_context;

    $terminate_allowed_values = [TRUE, FALSE];
    for ($i = 0; $i < $max_test_repeat_count; $i++) {
      $num = $i + 1;

      $terminate_value = $terminate_allowed_values[array_rand($terminate_allowed_values)];
      $this->drupalPostForm("admin/config/services/cloud/aws_cloud", [
                              'aws_cloud_instance_terminate' => $terminate_value,
                            ],
                            t('Save configuration'));
      $this->assertResponse(200);

      $this->createServerTemplate();
      $this->drupalGet("/clouds/design/server_template/$cloud_context/$num/launch");
      if ($terminate_value) {
        $this->assertFieldChecked('edit-terminate');
      }
      else {
        $this->assertNoFieldChecked('edit-terminate');
      }
    }
  }

  /**
   * Test creating an image from an instance
   */
  public function testImageCreationFromInstance() {
    $this->_testImageCreationFromInstance(AWS_CLOUD_INSTANCE_REPEAT_COUNT);
  }

  private function _testImageCreationFromInstance($max_test_repeat_count = 1) {
    $cloud_context = $this->cloud_context;

    for ($i = 0; $i < $max_test_repeat_count; $i ++) {
      // Setup server template and instance.
      $num = $i + 1;

      $this->createServerTemplate();
      $add = $this->createInstanceTestData();
      $this->addInstanceMockData($add[$i]['name'], $add[$i]['key_pair_name']);
      $this->drupalPostForm("/clouds/design/server_template/$cloud_context/$num/launch",
        [],
        t('Launch'));

      $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance @num', [
        '@num' => $i
      ]));
      $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));

      // Make sure instances are available.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/$num");
      $this->assertResponse(200, t('HTTP 200: Instance created.'));

      // Test image creation.
//      $image_id = $add[$i]['image_id'];
      $image_id = 'ami-' . $this->random->name(8, TRUE);
      $image_name = $this->random->name(8, TRUE);

      $image_params = [
        'image_name' => $image_name,
        'no_reboot' => 0,
      ];

      // Update the mock data then create the image.
      $this->updateImageCreationInMockData($image_id, $image_name, 'pending');
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/create_image",
        $image_params,
        t('Create Image'));

      $this->assertText(t("The AWS Cloud Instance @label (@image_id) has been created.", [
        '@image_id' => $image_id,
        '@label' => $add[$i]['name'],
      ]));

      // Make sure the image was created.  Status should be pending.
      // Click on the Image link from the image listing page.
      $this->clickLink($image_name);
      $this->assertResponse(200, t('HTTP 200: Image Entity.'));
      $this->assertText($image_id, t('Image Id is present'));
      $this->assertText('pending', t('Image created in pending state'));

      // Go back to listing page. Make sure there is no delete link.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertNoText('Delete', t('Cannot delete image in pending state'));

      // Update the image to 'available'.  Then delete the image
      $this->updateImageCreationInMockData($image_id, $image_name, 'available');

      // Run cron job to update images state.
      $key = \Drupal::state()->get('system.cron_key');
      $this->drupalGet('/cron/' . $key);
      $this->assertResponse(204);

      // Go back into the main image.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      // Click into the image. Make sure the status is now available.
      $this->clickLink($image_name);
      $this->assertText('available', t('Image status is available.'));

      // Go back to main listing page
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");

      // Delete the image.
      $this->clickLink('Delete');
      $this->drupalPostForm($this->getUrl(),
        [],
        t('Delete'));
      $this->assertResponse(200, t('HTTP 200: Delete', [
        '@num' => $i
      ]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Wanings'));

      // Make sure image is deleted
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertNoText($image_id, t('Image deleted'));

      // Test "Failed" Image.  Failed Images should be allowed to be deleted.

      // Reset the image_id and image_name variables.
      $image_id = 'ami-' . $this->random->name(8, TRUE);
      $image_name = $this->random->name(8, TRUE);

      $image_params = [
        'image_name' => $image_name,
        'no_reboot' => 0,
      ];

      // Update the image so it is in failed state.
      $this->updateImageCreationInMockData($image_id, $image_name, 'failed');
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/instance/$num/create_image",
        $image_params,
        t('Create Image'));

      $this->assertText(t("The AWS Cloud Instance @label (@image_id) has been created.", [
        '@image_id' => $image_id,
        '@label' => $add[$i]['name'],
      ]));

      // Go to the main image page.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      // Make sure the status is now failed.
      $this->clickLink($image_name);
      $this->assertText('failed', t('Image status is failed'));

      // Go to the main image page.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");

      // Delete the Failed image.
      $this->clickLink('Delete');
      $this->drupalPostForm($this->getUrl(),
        [],
        t('Delete'));
      $this->assertResponse(200, t('HTTP 200: Delete', [
        '@num' => $i
      ]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Wanings'));

      // Make sure image is deleted
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/image");
      $this->assertNoText($image_id, t('Image deleted'));

    }
  }

  /**
   * Test launching instances with schedule tags.
   */
  public function testCreateInstanceWithScheduleTag() {
    $this->_testCreateInstanceWithScheduleTag(AWS_CLOUD_INSTANCE_REPEAT_COUNT);
  }

  private function _testCreateInstanceWithScheduleTag($max_test_repeat_count = 1) {
    $cloud_context = $this->cloud_context;

    // Setup an arbitrary schedule in the configuration.
    // This is needed in the server template launch confirmation form.
    $schedule_value = $this->random->name(8, TRUE);

    $config = \Drupal::configFactory()->getEditable('aws_cloud.settings');
    $config->set('aws_cloud_scheduler_periods', $schedule_value)
      ->save();

    for ($i = 0; $i < $max_test_repeat_count; $i++) {
      $num = $i + 1;
      $this->createServerTemplate();

      // Launch a new Instance, with schedule information.
      $add = $this->createInstanceTestData();
      $this->addInstanceMockData($add[$i]['name'], $add[$i]['key_pair_name'], 'running', $schedule_value);

      $this->drupalPostForm("/clouds/design/server_template/$cloud_context/$num/launch",
        ['schedule' => $schedule_value],
        t('Launch'));

      $this->assertResponse(200, t('HTTP 200: Launch | A New Cloud Instance @num', [
        '@num' => $num
      ]));
      $this->assertNoText(t('Notice'), t('Launch | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Launch | Make sure w/o Warnings'));

      // Go to the instance page.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/instance/$num");
      $this->assertText($schedule_value, t('Schedule'));
    }
  }

  private function createInstanceTestData() {
    $data = [];

    for ($i = 0; $i < AWS_CLOUD_INSTANCE_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'              => "Instance #$num - " . date('Y/m/d - ') . $this->random->name(8, TRUE),
        'image_id'          => 'ami-' . $this->random->name(8, TRUE),
        'min_count'         => $num,
        'max_count'         => $num * 2,
        'key_pair_name'     => "key_pair-$num-" . $this->random->name(8, TRUE),
        'is_monitoring'     => 0,
        'availability_zone' => "us-west-$num",
        'security_groups[]'   => "security_group-$num-" . $this->random->name(8, TRUE),
        'instance_type'     => "t$num.micro",
        'kernel_id'         => 'aki-' . $this->random->name(8, TRUE),
        'ramdisk_id'        => 'ari-' . $this->random->name(8, TRUE),
        'user_data'         => "User Data #$num: " . $this->random->string(64, TRUE),
        'login_username'    => "user-$num" . date('-Y/m/d-') . $this->random->name(8, TRUE),
      ];
    }
    return $data;
  }

  private function createServerTemplate() {
    // Create image.
    $data = [];
    $image = entity_create('aws_cloud_image', $data);
    $image->set('ami_name', 'image1');
    $image->setImageId('ami-abcdef');
    $image->save();

    $data = [
      'type' => 'aws_cloud',
      'name' => 'test_template1',
    ];

    // Create template.
    $template = entity_create('cloud_server_template', $data);
    $template->setCloudContext($this->cloud_context);
    $template->field_test_only->value = '1';
    $template->field_max_count->value = 1;
    $template->field_min_count->value = 1;
    $template->field_monitoring->value = '0';
    $template->field_instance_type->value = 't1.micro';
    $template->field_image_id->target_id = 1;

    $template->save();
  }

  private function addInstanceMockData($name, $key_pair_name, $state = 'running', $schedule_value = '') {
    $mock_data = $this->getMockDataFromConfig();
    $vars = $this->getMockDataTemplateVars();
    $vars['name'] = $name;
    $vars['key_name'] = $key_pair_name;
    $vars['state'] = $state;
    $instance_mock_data_content = $this->getMockDataFileContent(get_class($this), $vars, '_instance');

    $instance_mock_data = Yaml::decode($instance_mock_data_content);
    // OwnerId and ReservationId need to be set.
    $mock_data['DescribeInstances']['Reservations'][0]['OwnerId'] = $this->random->name(8, TRUE);
    $mock_data['DescribeInstances']['Reservations'][0]['ReservationId'] = $this->random->name(8, TRUE);

    // Add Schedule information if available.
    if (!empty($schedule_value)) {
      $instance_mock_data['Tags'][1]['Key'] = 'Schedule';
      $instance_mock_data['Tags'][1]['Value'] = $schedule_value;
    }

    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][] = $instance_mock_data;
    $this->updateMockDataToConfig($mock_data);
  }

  private function deleteFirstInstanceInMockData() {
    $mock_data = $this->getMockDataFromConfig();
    $instances = $mock_data['DescribeInstances']['Reservations'][0]['Instances'];
    array_shift($instances);
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'] = $instances;
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateSecurityGroupsInMockData($security_group_name1, $security_group_name2) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][0]['SecurityGroups']
      = [['GroupName' => $security_group_name1], ['GroupName' => $security_group_name2]];
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateDescribeSecurityGroupsInMockData($security_groups) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeSecurityGroups']['SecurityGroups'] = $security_groups;
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateInstanceTypeInMockData($instance_type) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][0]['InstanceType']
      = $instance_type;
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateInstanceInMockData($instance_index, $name) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][$instance_index]['Tags'][0]['Value'] = $name;
    $this->updateMockDataToConfig($mock_data);
  }

  private function updateImageCreationInMockData($image_id, $image_name, $image_state) {
    $mock_data = $this->getMockDataFromConfig();
    $vars = [
      'image_id' => $image_id,
      'name' => $image_name,
      'state' => $image_state,
    ];
    // Unset DescribeImages so that the state can be updated.
    unset($mock_data['DescribeImages']);
    unset($mock_data['CreateImage']);
    $image_mock_data_content = $this->getMockDataFileContent('Drupal\Tests\aws_cloud\Functional\Ec2\ImageTest', $vars);
    $image_mock_data = Yaml::decode($image_mock_data_content);
    $image_mock_data['CreateImage']['ImageId'] = $image_id;
    $image_mock_data['DescribeImages']['Images'][0]['ImageId'] = $image_id;
    $this->updateMockDataToConfig(array_merge($image_mock_data, $mock_data));
  }

  private function createRandomSecurityGroups() {
    $random = $this->random;

    $security_groups = [];
    $count = rand(1, 10);
    for ($i = 0; $i < $count; $i++) {
      $security_groups[] = [
        'GroupId' => 'sg-' . $random->name(8, TRUE),
        'GroupName' => $random->name(10, TRUE),
      ];
    }

    return $security_groups;
  }

  private function updateScheduleTagInMockData($schedule_value) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][0]['Tags'][0]['Name'] = 'Schedule';
    $mock_data['DescribeInstances']['Reservations'][0]['Instances'][0]['Tags'][0]['Value'] = $schedule_value;
    $this->updateMockDataToConfig($mock_data);
  }
}
