<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Component\Utility\Random;

use Drupal\Tests\aws_cloud\Functional\Utils;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Uppated by yas 2016/05/28
// Updated by yas 2016/05/25
// Created by yas 2016/05/23.
define('AWS_CLOUD_SECURITY_GROUP_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Security Group.
 *
 * @group AWS Cloud
 */
class SecurityGroupTest extends AwsCloudTestCase {

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud security group',
      'add aws cloud security group',
      'view aws cloud security group',
      'edit aws cloud security group',
      'delete aws cloud security group',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'vpc_id' => 'vpc-' . $this->random->name(8, TRUE),
      'cidr_block' => Utils::getRandomCidr(),
      'group_id' => 'sg-' . $this->random->name(8, TRUE),
    ];
  }

  /**
   * Tests CRUD for Security Group information.
   */
  public function testSecurityGroup() {
    $cloud_context = $this->cloud_context;

    // List Security Group for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group");
    $this->assertResponse(200, t('List | HTTP 200: Security Group'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group/add");

    // Add a new Security Group.
    $add = $this->createSecurityGroupTestData();
    for ($i = 0; $i < AWS_CLOUD_SECURITY_GROUP_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/security_group/add",
                            $add[$i],
                            t('Save'));

      $this->assertResponse(200, t('Add | HTTP 200: A New AWS Cloud Security Group', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText($add[$i]['group_name'], t('Add | Key Pair: @group_name ', ['@group_name' => $add[$i]['group_name']]));
      $this->assertText(t('The AWS Cloud Security Group "@group_name" has been created.', [
                          '@group_name' => $add[$i]['group_name'],
                        ]),
                          t('Confirm Message') . ': '
                        . t('Add | The AWS Cloud Security Group "@group_name" has been created.', [
                          '@group_name' => $add[$i]['group_name'],
                        ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group");
      $this->assertResponse(200, t('Add | List | HTTP 200: List | Security Group #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));
      // 3 times.
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$i]['group_name'],
                        t('Add | List | Make sure w/ Listing: @group_name', [
                          '@group_name' => $add[$i]['group_name'],
                        ]));
      }
    }

    // Security Group doesn't have an edit operation.
    // Edit an Security Group information.
    $edit = $this->createSecurityGroupTestData(TRUE);
    for ($i = 0; $i < AWS_CLOUD_SECURITY_GROUP_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['description']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/security_group/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('Edit | HTTP 200: A New AWS Cloud Security Group #@num', array('@num' => $num)));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Security Group "@name" has been saved.', array(
                          '@name' => $edit[$i]['name'],
                        )),
                        t('Confirm Message') . ': '
                        . t('Edit | The AWS Cloud Security Group "@name" has been saved.', array(
                          '@name' => $edit[$i]['name'],
                        )));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Security Group #@num', array('@num' => $num)));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));

      $this->assertText($edit[$i]['name'], t('key_pair: @name ',
                        array('@name' => $edit[$i]['name'])));
    }

    // Delete Security Group.
    for ($i = 0; $i < AWS_CLOUD_SECURITY_GROUP_REPEAT_COUNT; $i++) {
      $num = $i + 1;
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group/$num/delete");
      $this->assertResponse(200, t('Delete | HTTP 200: Security Group #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/security_group/$num/delete",
                            [],
                            t('Delete'));

      $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Security Group #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->assertText($add[$i]['group_name'], t('Group Name: @group_name ', ['@group_name' => $add[$i]['group_name']]));
      $this->assertText(t('The AWS Cloud Security Group "@group_name" has been deleted.', [
                          '@group_name' => $add[$i]['group_name'],
                        ]),
                          t('Confirm Message') . ': '
                        . t('Delete | The AWS Cloud Security Group "@group_name" has been deleted.', [
                          '@group_name' => $add[$i]['group_name'],
                        ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/security_group");
      $this->assertResponse(200, t('HTTP 200: Delete | Security Group #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));
    }
  }

  /**
   *
   */
  private function createSecurityGroupTestData($isEdit = FALSE) {
    $data = [];

    for ($i = 0; $i < AWS_CLOUD_SECURITY_GROUP_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'description' => "Description #$num - " . $this->random->name(64, TRUE),
      ];

      if ($isEdit) {
        $data[$i]['name'] = "group-name-#$num - " . $this->random->name(15, TRUE);
      }
      else {
        $data[$i]['group_name'] = "group-name-#$num - " . $this->random->name(15, TRUE);
      }
    }
    return $data;
  }

}
