<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Component\Utility\Random;

use Drupal\Tests\aws_cloud\Functional\Utils;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/28
// Updated by yas 2016/05/25
// Updated by yas 2016/05/24
// Created by yas 2016/05/23.
define('AWS_CLOUD_SNAPSHOT_REPEAT_COUNT', 3);

/**
 * Tests AWS Cloud Snapshot.
 *
 * @group AWS Cloud
 */
class SnapshotTest extends AwsCloudTestCase {

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud snapshot',
      'add aws cloud snapshot',
      'view aws cloud snapshot',
      'edit aws cloud snapshot',
      'delete aws cloud snapshot',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'snapshot_id' => 'snap-' . $this->random->name(17, TRUE),
      'cidr_block' => Utils::getRandomCidr(),
      'group_id' => 'sg-' . $this->random->name(8, TRUE),
      'start_time' => date('c'),
    ];
  }

  /**
   * Tests CRUD for Snapshot information.
   */
  public function testSnapshot() {
    $cloud_context = $this->cloud_context;

    // List Snapshot for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
    $this->assertResponse(200, t('List | HTTP 200: Snapshot'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Add a new Snapshot.
    $add = $this->createSnapshotTestData();
    for ($i = 0; $i < AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/add",
                            $add[$i],
                            t('Save'));

      $this->assertResponse(200, t('HTTP 200: Add | A New AWS Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Snapshot "@name', [
                        '@name' => $add[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Snapshot "@name" has been created.', [
                        '@name' => $add[$i]['name'],
                      ]));
      $this->assertText($add[$i]['name'],
                     t('key_pair: @name ', [
                       '@name' => $add[$i]['name'],
                     ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('HTTP 200: List | Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));
    }

    // Edit an Snapshot information.
    $edit = $this->createSnapshotTestData();
    for ($i = 0; $i < AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['volume_id']);
      unset($edit[$i]['description']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('Edit | HTTP 200: A New AWS Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Snapshot "@name" has been saved.', [
        '@name' => $edit[$i]['name'],
      ]),
                        t('Confirm Message') . ': '
                      . t('The AWS Cloud Snapshot "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));

      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $edit[$i]['name'],
                        ]));
      }
    }

    // Delete Snapshot.
    for ($i = 0; $i < AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot/$num/delete");
      $this->assertResponse(200, t('Delete | HTTP 200: Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/snapshot/$num/delete",
                            [],
                            t('Delete'));

      $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Snapshot #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Snapshot "@name" has been deleted.', ['@name' => $edit[$i]['name']]),
        t('Confirm Message') . ': ' . t('Delete | The AWS Cloud Snapshot "@name" has been deleted.', ['@name' => $edit[$i]['name']]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/snapshot");
      $this->assertResponse(200, t('Delete | HTTP 200: Snapshot', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | List | Make sure w/o Warnings'));
    }
  }

  /**
   *
   */
  private function createSnapshotTestData() {
    $data = [];

    for ($i = 0; $i < AWS_CLOUD_SNAPSHOT_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'        => "Name #$num - " . $this->random->name(32, TRUE),
        'volume_id'   => "vol-" . $this->random->name(8, TRUE),
        'description' => "Description #$num - " . $this->random->name(64, TRUE),
      ];
    }
    return $data;
  }

}
