<?php

namespace Drupal\Tests\aws_cloud\Functional\Ec2;

use Drupal\Component\Utility\Random;

// Updated by yas 2016/06/23
// Updated by yas 2016/06/05
// Updated by yas 2016/06/02
// Updated by yas 2016/05/31
// Updated by yas 2016/05/29
// Updated by yas 2016/05/25
// Created by yas 2016/05/23.

/**
 * Tests AWS Cloud Volume.
 *
 * @group AWS Cloud
 */
class VolumeTest extends AwsCloudTestCase {

  const AWS_CLOUD_VOLUME_REPEAT_COUNT = 3;

  /**
   * {@inheritdoc}
   */
  protected function getPermissions() {
    return [
      'list aws cloud volume',
      'add aws cloud volume',
      'view aws cloud volume',
      'edit aws cloud volume',
      'delete aws cloud volume',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function getMockDataTemplateVars() {
    return [
      'volume_id' => 'vol-' . $this->random->name(17, TRUE),
      'create_time' => date('c'),
    ];
  }

  /**
   * Tests CRUD for Volume information.
   */
  public function testVolume() {
    $cloud_context = $this->cloud_context;

    // List Volume for Amazon EC2.
    $this->drupalGet("/clouds/aws_cloud/$cloud_context/volume");
    $this->assertResponse(200, t('HTTP 200: List'));
    $this->assertNoText(t('Notice'), t('List | Make sure w/o Notice'));
    $this->assertNoText(t('warning'), t('List | Make sure w/o Warnings'));

    // Add a new Volume.
    $delete_count = 0;
    $add = $this->createVolumeTestData();
    for ($i = 0; $i < self::AWS_CLOUD_VOLUME_REPEAT_COUNT; $i++) {
      $this->reloadMockData();

      $state = $this->createRandomState();
      $this->updateCreateVolumeInMockData($state);
      if ($state != 'in-use') {
        $delete_count++;
      }

      $num = $i + 1;

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/volume/add",
                            $add[$i],
                            t('Save'));

      $this->assertResponse(200, t('Add | HTTP 200: A New AWS Cloud Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | Make sure w/o Warnings'));
      $this->assertText($add[$i]['name'], t('Add | Volume: @name ', ['@name' => $add[$i]['name']]));
      $this->assertText(t('The AWS Cloud Volume "@name', [
                        '@name' => $add[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Add | The AWS Cloud Volume "@name" has been created.', [
                        '@name' => $add[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/volume");
      $this->assertResponse(200, t('HTTP 200: Add | List | Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Add | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Add | List | Make sure w/o Warnings'));
      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($add[$i]['name'],
                        t('Add | List | Make sure w/ Listing: @name', [
                          '@name' => $add[$i]['name'],
                        ]));
      }

      // Assert delete link count.
      if ($delete_count > 0) {
        $this->assertLink(t('Delete'), $delete_count - 1);
      }
    }

    // Edit an Volume information.
    $edit = $this->createVolumeTestData();
    for ($i = 0; $i < self::AWS_CLOUD_VOLUME_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      unset($edit[$i]['snapshot_id']);
      unset($edit[$i]['size']);
      unset($edit[$i]['availability_zone']);
      unset($edit[$i]['iops']);
      unset($edit[$i]['encrypted']);
      unset($edit[$i]['volume_type']);

      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/volume/$num/edit",
                            $edit[$i],
                            t('Save'));

      $this->assertResponse(200, t('HTTP 200: Edit | A New AWS Cloud Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | Make sure w/o Warnings'));
      $this->assertText(t('The AWS Cloud Volume "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Edit | The AWS Cloud Volume "@name" has been saved.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/volume");
      $this->assertResponse(200, t('Edit | List | HTTP 200: Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Edit | List | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Edit | List | Make sure w/o Warnings'));

      for ($j = 0; $j < $i + 1; $j++) {
        $this->assertText($edit[$i]['name'],
                        t('Edit | List | Make sure w/ Listing: @name', [
                          '@name' => $edit[$i]['name'],
                        ]));
      }
    }

    // Delete Volume.
    for ($i = 0; $i < self::AWS_CLOUD_VOLUME_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      $this->drupalGet("/clouds/aws_cloud/$cloud_context/volume/$num/delete");
      $this->assertResponse(200, t('Delete | HTTP 200: Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->drupalPostForm("/clouds/aws_cloud/$cloud_context/volume/$num/delete",
                            [],
                            t('Delete'));

      $this->assertResponse(200, t('Delete | HTTP 200: The Cloud Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
      $this->assertText($edit[$i]['name'], t('Delete | Name: @name ', ['@name' => $edit[$i]['name']]));
      $this->assertText(t('The AWS Cloud Volume "@name" has been deleted.', [
                        '@name' => $edit[$i]['name'],
                      ]),
                        t('Confirm Message') . ': '
                      . t('Delete | The AWS Cloud Volume "@name" has been deleted.', [
                        '@name' => $edit[$i]['name'],
                      ]));

      // Make sure listing.
      $this->drupalGet("/clouds/aws_cloud/$cloud_context/volume");
      $this->assertResponse(200, t('Delete | HTTP 200: Volume #@num', ['@num' => $num]));
      $this->assertNoText(t('Notice'), t('Delete | Make sure w/o Notice'));
      $this->assertNoText(t('warning'), t('Delete | Make sure w/o Warnings'));
    }
  }

  private function createVolumeTestData() {
    $data = [];

    for ($i = 0; $i < self::AWS_CLOUD_VOLUME_REPEAT_COUNT; $i++) {

      $num = $i + 1;

      // Input Fields.
      $data[$i] = [
        'name'              => "volume-name #$num - " . $this->random->name(32, TRUE),
        'snapshot_id'       => "snapshot-id-$num-" . $this->random->name(8, TRUE),
        'size'              => $num * 10,
        'availability_zone' => "us-west-$num",
        'iops'              => $num * 1000,
        'encrypted'         => $num % 2,
        'volume_type'       => 'io1',
      ];
    }
    return $data;
  }

  private function createRandomState() {
    $states = ['creating', 'in-use'];
    return $states[array_rand($states)];
  }

  private function updateCreateVolumeInMockData($state) {
    $mock_data = $this->getMockDataFromConfig();
    $mock_data['CreateVolume']['State'] = $state;
    $this->updateMockDataToConfig($mock_data);
  }

}
